@@ -1,32 +1,52 @@
 use crate::utils::*;
-use crate::{free, Alpm, AlpmList, AlpmListMut, AsRawAlpmList, Db, Package, Ver};
+use crate::{free, Alpm, AlpmList, AlpmListMut, Db, IntoRawAlpmList, Package, Ver};
 
 use alpm_sys::alpm_depmod_t::*;
 use alpm_sys::*;
 
 use std::ffi::{c_void, CString};
 use std::fmt;
-use std::hash::{Hash, Hasher};
 use std::marker::PhantomData;
 use std::mem::transmute;
 
-#[derive(Debug)]
 pub struct Dep<'a> {
     pub(crate) inner: *mut alpm_depend_t,
     pub(crate) phantom: PhantomData<&'a ()>,
 }
 
+impl<'a> fmt::Debug for Dep<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Dep")
+            .field("name", &self.name())
+            .field("version", &self.version())
+            .field("desc", &self.desc())
+            .field("depmod", &self.depmod())
+            .field("name_hash", &self.name_hash())
+            .finish()
+    }
+}
+
 unsafe impl<'a> Send for Dep<'a> {}
 unsafe impl<'a> Sync for Dep<'a> {}
 
-#[derive(Debug, Hash, PartialEq, Eq)]
+#[derive(PartialEq)]
 pub struct Depend {
     dep: Dep<'static>,
 }
 
 impl Clone for Depend {
     fn clone(&self) -> Self {
-        Depend::new(self.to_string())
+        let ptr = unsafe { alpm_dep_compute_string(self.inner) };
+        assert!(!ptr.is_null(), "failed to compute string for dep");
+        let dep = unsafe { alpm_dep_from_string(ptr) };
+        assert!(!dep.is_null(), "failed to create dep from string");
+        unsafe { Depend::from_ptr(dep) }
+    }
+}
+
+impl fmt::Debug for Depend {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Debug::fmt(&self.as_dep(), f)
     }
 }
 
@@ -45,24 +65,24 @@ impl std::ops::Deref for Depend {
 }
 
 pub trait AsDep {
-    fn as_dep(&self) -> &Dep;
+    fn as_dep<'a>(&'a self) -> Dep<'a>;
 }
 
-impl AsDep for Depend {
-    fn as_dep(&self) -> &Dep {
-        &self.dep
+impl<'a> AsDep for Depend {
+    fn as_dep<'b>(&'b self) -> Dep<'b> {
+        self.dep()
     }
 }
 
 impl<'a> AsDep for Dep<'a> {
-    fn as_dep(&self) -> &Dep {
-        self
+    fn as_dep<'b>(&'b self) -> Dep<'b> {
+        self.dep()
     }
 }
 
 impl<'a> AsDep for &Dep<'a> {
-    fn as_dep(&self) -> &Dep {
-        self
+    fn as_dep<'b>(&'b self) -> Dep<'b> {
+        self.dep()
     }
 }
 
@@ -72,14 +92,6 @@ impl Drop for Depend {
     }
 }
 
-impl<'a> Hash for Dep<'a> {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.name().hash(state);
-        self.depmod().hash(state);
-        self.version().hash(state);
-    }
-}
-
 impl<'a> PartialEq for Dep<'a> {
     fn eq(&self, other: &Self) -> bool {
         self.name() == other.name()
@@ -89,24 +101,24 @@ impl<'a> PartialEq for Dep<'a> {
     }
 }
 
-impl<'a> Eq for Dep<'a> {}
-
 impl<'a> fmt::Display for Dep<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         unsafe {
             let cs = alpm_dep_compute_string(self.inner);
+            assert!(!cs.is_null(), "failed to compute string for dep");
             let s = from_cstr(cs);
-            let err = write!(f, "{}", s);
+            let err = f.write_str(s);
             free(cs as *mut c_void);
             err
         }
     }
 }
 
-impl<'a> Into<Vec<u8>> for Dep<'a> {
-    fn into(self) -> Vec<u8> {
+impl<'a> From<Dep<'a>> for Vec<u8> {
+    fn from(dep: Dep<'a>) -> Vec<u8> {
         unsafe {
-            let cs = alpm_dep_compute_string(self.inner);
+            let cs = alpm_dep_compute_string(dep.inner);
+            assert!(!cs.is_null(), "failed to compute string for dep");
             let s = std::ffi::CStr::from_ptr(cs);
             let s = s.to_bytes().to_vec();
             free(cs as *mut c_void);
@@ -119,6 +131,7 @@ impl Depend {
     pub fn new<S: Into<Vec<u8>>>(s: S) -> Depend {
         let s = CString::new(s).unwrap();
         let dep = unsafe { alpm_dep_from_string(s.as_ptr()) };
+        assert!(!dep.is_null(), "failed to create dep from string");
 
         Depend {
             dep: Dep {
@@ -139,6 +152,13 @@ impl Depend {
 }
 
 impl<'a> Dep<'a> {
+    pub fn dep<'b>(&'b self) -> Dep<'b> {
+        Dep {
+            inner: self.inner,
+            phantom: PhantomData,
+        }
+    }
+
     pub fn to_depend(&self) -> Depend {
         Depend::new(self.to_string())
     }
@@ -155,8 +175,8 @@ impl<'a> Dep<'a> {
         Ver::from_ptr((*self.inner).version)
     }
 
-    pub fn desc(&self) -> &'a str {
-        unsafe { from_cstr((*self.inner).desc) }
+    pub fn desc(&self) -> Option<&'a str> {
+        unsafe { from_cstr_optional((*self.inner).desc) }
     }
 
     pub fn name_hash(&self) -> u64 {
@@ -187,7 +207,7 @@ impl<'a> Dep<'a> {
     }
 }
 
-#[derive(Debug, Eq, PartialEq, Copy, Clone, Ord, PartialOrd, Hash)]
+#[derive(Debug, Eq, PartialEq, Copy, Clone, PartialOrd)]
 pub enum DepModVer<'a> {
     Any,
     Eq(&'a Ver),
@@ -230,12 +250,21 @@ pub enum DepMod {
 unsafe impl<'a> Send for DepMissing<'a> {}
 unsafe impl<'a> Sync for DepMissing<'a> {}
 
-#[derive(Debug)]
 pub struct DepMissing<'a> {
     pub(crate) inner: *mut alpm_depmissing_t,
     pub(crate) phantom: PhantomData<&'a ()>,
 }
 
+impl<'a> fmt::Debug for DepMissing<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("DepMissing")
+            .field("target", &self.target())
+            .field("depend", &self.depend())
+            .field("causing_pkg", &self.causing_pkg())
+            .finish()
+    }
+}
+
 impl std::ops::Deref for DependMissing {
     type Target = DepMissing<'static>;
 
@@ -244,11 +273,16 @@ impl std::ops::Deref for DependMissing {
     }
 }
 
-#[derive(Debug)]
 pub struct DependMissing {
     pub(crate) inner: DepMissing<'static>,
 }
 
+impl fmt::Debug for DependMissing {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Debug::fmt(&self.inner, f)
+    }
+}
+
 impl Drop for DependMissing {
     fn drop(&mut self) {
         unsafe { alpm_depmissing_free(self.inner.inner) }
@@ -300,16 +334,16 @@ impl<'a> AlpmList<'a, Package<'a>> {
 impl Alpm {
     pub fn check_deps<'a>(
         &self,
-        pkgs: impl AsRawAlpmList<'a, Package<'a>>,
-        rem: impl AsRawAlpmList<'a, Package<'a>>,
-        upgrade: impl AsRawAlpmList<'a, Package<'a>>,
+        pkgs: impl IntoRawAlpmList<'a, Package<'a>>,
+        rem: impl IntoRawAlpmList<'a, Package<'a>>,
+        upgrade: impl IntoRawAlpmList<'a, Package<'a>>,
         reverse_deps: bool,
     ) -> AlpmListMut<DependMissing> {
         let reverse_deps = if reverse_deps { 1 } else { 0 };
 
-        let pkgs = unsafe { pkgs.as_raw_alpm_list() };
-        let rem = unsafe { rem.as_raw_alpm_list() };
-        let upgrade = unsafe { upgrade.as_raw_alpm_list() };
+        let pkgs = unsafe { pkgs.into_raw_alpm_list() };
+        let rem = unsafe { rem.into_raw_alpm_list() };
+        let upgrade = unsafe { upgrade.into_raw_alpm_list() };
 
         let ret = unsafe {
             alpm_checkdeps(
