@@ -39,7 +39,6 @@ pub enum Error {
     TransNotNull = ALPM_ERR_TRANS_NOT_NULL as u32,
     TransNull = ALPM_ERR_TRANS_NULL as u32,
     TransDupTarget = ALPM_ERR_TRANS_DUP_TARGET as u32,
-    #[cfg(feature = "git")]
     TransDupFileName = ALPM_ERR_TRANS_DUP_FILENAME as u32,
     TransNotInitialized = ALPM_ERR_TRANS_NOT_INITIALIZED as u32,
     TransNotPrepared = ALPM_ERR_TRANS_NOT_PREPARED as u32,
@@ -57,7 +56,6 @@ pub enum Error {
     PkgCantRemove = ALPM_ERR_PKG_CANT_REMOVE as u32,
     PkgInvalidName = ALPM_ERR_PKG_INVALID_NAME as u32,
     PkgInvalidArch = ALPM_ERR_PKG_INVALID_ARCH as u32,
-    PkgRepoNotFound = ALPM_ERR_PKG_REPO_NOT_FOUND as u32,
     SigMissing = ALPM_ERR_SIG_MISSING as u32,
     SigInvalid = ALPM_ERR_SIG_INVALID as u32,
     UnsatisfiedDeps = ALPM_ERR_UNSATISFIED_DEPS as u32,
