@@ -4,20 +4,30 @@ use crate::Result;
 use alpm_sys::*;
 
 use std::ffi::CString;
+use std::fmt;
 use std::slice;
 
-#[derive(Debug)]
 pub struct File {
     inner: alpm_file_t,
 }
 
+impl fmt::Debug for File {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("File")
+            .field("name", &self.name())
+            .field("size", &self.size())
+            .field("mode", &self.mode())
+            .finish()
+    }
+}
+
 impl File {
     pub fn name(&self) -> &str {
         unsafe { from_cstr(self.inner.name) }
     }
 
     pub fn size(&self) -> i64 {
-        #[allow(clippy::identity_conversion)]
+        #[allow(clippy::useless_conversion)]
         self.inner.size.into()
     }
 
@@ -26,11 +36,16 @@ impl File {
     }
 }
 
-#[derive(Debug)]
 pub struct FileList {
     pub(crate) inner: alpm_filelist_t,
 }
 
+impl fmt::Debug for FileList {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_list().entries(self.files()).finish()
+    }
+}
+
 impl FileList {
     pub fn files(&self) -> &[File] {
         unsafe { slice::from_raw_parts(self.inner.files as *const File, self.inner.count) }
