@@ -1,19 +1,11 @@
 use crate::utils::*;
 use crate::{
-    Alpm, AlpmList, AsRawAlpmList, Db, DbMut, Dep, Depend, DownloadCb, EventCb, FetchCb, LogCb,
-    Match, ProgressCb, QuestionCb, Result, SigLevel,
+    Alpm, AlpmList, AsDep, Db, DbMut, Dep, Depend, IntoRawAlpmList, Match, Result, SigLevel,
 };
 
-#[cfg(not(feature = "git"))]
-use crate::TotalDownloadCb;
-
+use alpm_sys::*;
 use std::cmp::Ordering;
 use std::ffi::CString;
-use std::marker::PhantomData;
-#[cfg(feature = "git")]
-use std::ptr;
-
-use alpm_sys::*;
 
 impl Alpm {
     pub fn as_alpm_handle_t(&self) -> *mut alpm_handle_t {
@@ -85,12 +77,6 @@ impl Alpm {
         AlpmList::from_parts(self, list)
     }
 
-    #[cfg(not(feature = "git"))]
-    pub fn arch(&self) -> &str {
-        unsafe { from_cstr(alpm_option_get_arch(self.handle)) }
-    }
-
-    #[cfg(feature = "git")]
     pub fn architectures(&self) -> AlpmList<'_, &str> {
         let list = unsafe { alpm_option_get_architectures(self.handle) };
         AlpmList::from_parts(self, list)
@@ -110,8 +96,8 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_hookdirs<'a, T: AsRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_hookdirs<'a, T: IntoRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_hookdirs(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -132,8 +118,8 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_cachedirs<'a, T: AsRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_cachedirs<'a, T: IntoRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_cachedirs(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -171,8 +157,8 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_noupgrades<'a, T: AsRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_noupgrades<'a, T: IntoRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_noupgrades(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -204,8 +190,8 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_noextracts<'a, T: AsRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_noextracts<'a, T: IntoRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_noextracts(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -237,8 +223,8 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_ignorepkgs<'a, T: AsRawAlpmList<'a, String>>(&mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_ignorepkgs<'a, T: IntoRawAlpmList<'a, String>>(&mut self, list: T) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_ignorepkgs(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -259,8 +245,11 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_ignoregroups<'a, T: AsRawAlpmList<'a, String>>(&'a mut self, list: T) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+    pub fn set_ignoregroups<'a, T: IntoRawAlpmList<'a, String>>(
+        &'a mut self,
+        list: T,
+    ) -> Result<()> {
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_ignoregroups(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -281,11 +270,11 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    pub fn set_overwrite_files<'a, T: AsRawAlpmList<'a, String>>(
+    pub fn set_overwrite_files<'a, T: IntoRawAlpmList<'a, String>>(
         &'a mut self,
         list: T,
     ) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_overwrite_files(self.handle, list.list()) };
         self.check_ret(ret)
     }
@@ -305,19 +294,17 @@ impl Alpm {
         self.check_ret(ret)
     }
 
-    // Broken in stable
-    #[cfg(feature = "git")]
-    pub fn set_assume_installed<'a, T: AsRawAlpmList<'a, Dep<'a>>>(
+    pub fn set_assume_installed<'a, T: IntoRawAlpmList<'a, Dep<'a>>>(
         &'a mut self,
         list: T,
     ) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_assumeinstalled(self.handle, list.list()) };
         self.check_ret(ret)
     }
 
-    pub fn remove_assume_installed(&mut self, s: &Dep) -> Result<bool> {
-        let ret = unsafe { alpm_option_remove_assumeinstalled(self.handle, s.inner) };
+    pub fn remove_assume_installed<D: AsDep>(&mut self, s: D) -> Result<bool> {
+        let ret = unsafe { alpm_option_remove_assumeinstalled(self.handle, s.as_dep().inner) };
         if ret == 1 {
             Ok(true)
         } else {
@@ -325,30 +312,21 @@ impl Alpm {
         }
     }
 
-    #[cfg(not(feature = "git"))]
-    pub fn set_arch<S: Into<Vec<u8>>>(&self, s: S) {
-        let s = CString::new(s).unwrap();
-        unsafe { alpm_option_set_arch(self.handle, s.as_ptr()) };
-    }
-
-    #[cfg(feature = "git")]
     pub fn add_architecture<S: Into<Vec<u8>>>(&mut self, s: S) -> Result<()> {
         let s = CString::new(s).unwrap();
         let ret = unsafe { alpm_option_add_architecture(self.handle, s.as_ptr()) };
         self.check_ret(ret)
     }
 
-    #[cfg(feature = "git")]
-    pub fn set_architectures<'a, T: AsRawAlpmList<'a, String>>(
+    pub fn set_architectures<'a, T: IntoRawAlpmList<'a, String>>(
         &'a mut self,
         list: T,
     ) -> Result<()> {
-        let list = unsafe { list.as_raw_alpm_list() };
+        let list = unsafe { list.into_raw_alpm_list() };
         let ret = unsafe { alpm_option_set_architectures(self.handle, list.list()) };
         self.check_ret(ret)
     }
 
-    #[cfg(feature = "git")]
     pub fn remove_architecture<S: Into<Vec<u8>>>(&mut self, s: S) -> Result<bool> {
         let s = CString::new(s).unwrap();
         let ret = unsafe { alpm_option_remove_architecture(self.handle, s.as_ptr()) };
@@ -419,131 +397,9 @@ impl Alpm {
         unsafe { alpm_option_set_disable_dl_timeout(self.handle, b) };
     }
 
-    #[cfg(feature = "git")]
     pub fn set_parallel_downloads(&self, n: u32) {
         unsafe { alpm_option_set_parallel_downloads(self.handle, n) };
     }
-
-    pub fn log_cb(&self) -> LogCb {
-        LogCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_logcb(self.handle) },
-        }
-    }
-
-    pub fn dl_cb(&self) -> DownloadCb {
-        DownloadCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_dlcb(self.handle) },
-        }
-    }
-
-    pub fn fetch_cb(&self) -> FetchCb {
-        FetchCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_fetchcb(self.handle) },
-        }
-    }
-
-    #[cfg(not(feature = "git"))]
-    pub fn totaldl_cb(&self) -> TotalDownloadCb {
-        TotalDownloadCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_totaldlcb(self.handle) },
-        }
-    }
-
-    pub fn event_cb(&self) -> EventCb {
-        EventCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_eventcb(self.handle) },
-        }
-    }
-
-    pub fn question_cb(&self) -> QuestionCb {
-        QuestionCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_questioncb(self.handle) },
-        }
-    }
-
-    pub fn progress_cb(&self) -> ProgressCb {
-        ProgressCb {
-            marker: PhantomData,
-            cb: unsafe { alpm_option_get_progresscb(self.handle) },
-        }
-    }
-
-    pub fn set_log_cb(&self, cb: LogCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_logcb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_logcb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
-
-    pub fn set_dl_cb(&self, cb: DownloadCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_dlcb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_dlcb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
-
-    pub fn set_fetch_cb(&self, cb: FetchCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_fetchcb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_fetchcb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
-
-    #[cfg(not(feature = "git"))]
-    pub fn set_totaldl_cb(&self, cb: TotalDownloadCb) {
-        unsafe { alpm_option_set_totaldlcb(self.handle, cb.cb) };
-    }
-
-    pub fn set_event_cb(&self, cb: EventCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_eventcb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_eventcb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
-
-    pub fn set_question_cb(&self, cb: QuestionCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_questioncb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_questioncb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
-
-    pub fn set_progress_cb(&self, cb: ProgressCb) {
-        #[cfg(not(feature = "git"))]
-        unsafe {
-            alpm_option_set_progresscb(self.handle, cb.cb)
-        };
-        #[cfg(feature = "git")]
-        unsafe {
-            alpm_option_set_progresscb(self.handle, cb.cb, ptr::null_mut())
-        };
-    }
 }
 
 #[cfg(test)]
