@@ -1,9 +1,10 @@
 use crate::{
     free, Alpm, Backup, Conflict, Db, DbMut, Dep, DepMissing, Depend, DependMissing, FileConflict,
-    Group, LoadedPackage, OwnedConflict, OwnedFileConflict, Package,
+    Group, LoadedPackage, OwnedConflict, OwnedFileConflict, Package, Pkg,
 };
 
 use std::ffi::{c_void, CStr};
+use std::fmt;
 use std::iter::{ExactSizeIterator, Iterator};
 use std::marker::PhantomData;
 use std::mem::ManuallyDrop;
@@ -17,9 +18,11 @@ extern "C" {
 }
 
 pub unsafe trait IntoAlpmListItem<'a, 'b> {
-    type Borrow;
-    unsafe fn into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self;
-    unsafe fn as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow;
+    type Borrow: fmt::Debug;
+    #[doc(hidden)]
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self;
+    #[doc(hidden)]
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow;
 }
 
 pub unsafe trait AsAlpmListItemPtr<'a> {
@@ -80,17 +83,19 @@ where
     }
 }
 
-pub trait AsRawAlpmList<'a, T>
+pub trait IntoRawAlpmList<'a, T>
 where
     T: AsAlpmListItemPtr<'a>,
 {
+    #[doc(hidden)]
     type Drop: Bool;
-    unsafe fn as_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop>;
+    #[doc(hidden)]
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop>;
 }
 
-impl<'a> AsRawAlpmList<'a, Package<'a>> for AlpmList<'a, LoadedPackage<'a>> {
+impl<'a> IntoRawAlpmList<'a, Pkg<'a>> for AlpmList<'a, LoadedPackage<'a>> {
     type Drop = False;
-    unsafe fn as_raw_alpm_list(self) -> RawAlpmList<'a, Package<'a>, Self::Drop> {
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, Pkg<'a>, Self::Drop> {
         RawAlpmList {
             list: self.list,
             _marker1: PhantomData,
@@ -99,12 +104,23 @@ impl<'a> AsRawAlpmList<'a, Package<'a>> for AlpmList<'a, LoadedPackage<'a>> {
     }
 }
 
-impl<'a, T> AsRawAlpmList<'a, T> for AlpmList<'a, T>
+impl<'a> IntoRawAlpmList<'a, Pkg<'a>> for AlpmList<'a, Package<'a>> {
+    type Drop = False;
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, Pkg<'a>, Self::Drop> {
+        RawAlpmList {
+            list: self.list,
+            _marker1: PhantomData,
+            _marker2: PhantomData,
+        }
+    }
+}
+
+impl<'a, T> IntoRawAlpmList<'a, T> for AlpmList<'a, T>
 where
     T: AsAlpmListItemPtr<'a>,
 {
     type Drop = False;
-    unsafe fn as_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop> {
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop> {
         RawAlpmList {
             list: self.list,
             _marker1: PhantomData,
@@ -113,12 +129,12 @@ where
     }
 }
 
-impl<'a, T> AsRawAlpmList<'a, T> for &AlpmListMut<'a, T>
+impl<'a, T> IntoRawAlpmList<'a, T> for &AlpmListMut<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b> + AsAlpmListItemPtr<'a>,
 {
     type Drop = False;
-    unsafe fn as_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop> {
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop> {
         RawAlpmList {
             list: self.list.list,
             _marker1: PhantomData,
@@ -127,14 +143,24 @@ where
     }
 }
 
-impl<'a, T, I> AsRawAlpmList<'a, T::Output> for I
+impl<'a, T, D: Bool> IntoRawAlpmList<'a, T> for RawAlpmList<'a, T, D>
+where
+    T: AsAlpmListItemPtr<'a>,
+{
+    type Drop = D;
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop> {
+        self
+    }
+}
+
+impl<'a, T, I> IntoRawAlpmList<'a, T::Output> for I
 where
     I: Iterator<Item = T>,
     T: AsAlpmListItemPtr<'a>,
     T::Output: AsAlpmListItemPtr<'a>,
 {
     type Drop = True;
-    unsafe fn as_raw_alpm_list(self) -> RawAlpmList<'a, T::Output, Self::Drop> {
+    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T::Output, Self::Drop> {
         let mut list = ptr::null_mut();
 
         for item in self {
@@ -152,13 +178,22 @@ where
     }
 }
 
-#[derive(Debug)]
 pub struct AlpmList<'a, T> {
     pub(crate) handle: &'a Alpm,
     pub(crate) list: *mut alpm_list_t,
     pub(crate) _marker: PhantomData<T>,
 }
 
+impl<'a, T> fmt::Debug for AlpmList<'a, T>
+where
+    for<'b> T: IntoAlpmListItem<'a, 'b>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str("AlpmList ")?;
+        f.debug_list().entries(self).finish()
+    }
+}
+
 impl<'a, T> Clone for AlpmList<'a, T> {
     fn clone(&self) -> Self {
         AlpmList {
@@ -171,7 +206,6 @@ impl<'a, T> Clone for AlpmList<'a, T> {
 
 impl<'a, T> Copy for AlpmList<'a, T> {}
 
-#[derive(Debug)]
 pub struct AlpmListMut<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
@@ -179,6 +213,15 @@ where
     list: AlpmList<'a, T>,
 }
 
+impl<'a, T> fmt::Debug for AlpmListMut<'a, T>
+where
+    for<'b> T: IntoAlpmListItem<'a, 'b>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Debug::fmt(&self.as_list(), f)
+    }
+}
+
 impl<'a, T> std::ops::Deref for AlpmListMut<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
@@ -199,7 +242,7 @@ where
         let mut curr = list;
 
         while !curr.is_null() {
-            let item = unsafe { T::into_alpm_list_item(self.handle, (*curr).data) };
+            let item = unsafe { T::ptr_into_alpm_list_item(self.handle, (*curr).data) };
             drop(item);
             curr = unsafe { (*curr).next };
         }
@@ -224,7 +267,7 @@ where
         if self.is_empty() {
             None
         } else {
-            unsafe { Some(T::as_alpm_list_item(self.handle, (*self.list).data)) }
+            unsafe { Some(T::ptr_as_alpm_list_item(self.handle, (*self.list).data)) }
         }
     }
 
@@ -233,7 +276,7 @@ where
         if item.is_null() {
             None
         } else {
-            unsafe { Some(T::as_alpm_list_item(self.handle, (*item).data)) }
+            unsafe { Some(T::ptr_as_alpm_list_item(self.handle, (*item).data)) }
         }
     }
 
@@ -252,7 +295,7 @@ impl<'a, T> AlpmList<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
 {
-    pub fn to_list(&self) -> AlpmListMut<'a, T> {
+    pub fn to_list_mut(&self) -> AlpmListMut<'a, T> {
         let list = unsafe { alpm_list_copy(self.list) };
         AlpmListMut {
             list: AlpmList::from_parts(self.handle, list),
@@ -302,7 +345,7 @@ where
         let mut curr = list;
 
         while !curr.is_null() {
-            let item = unsafe { T::into_alpm_list_item(self.handle, (*curr).data) };
+            let item = unsafe { T::ptr_into_alpm_list_item(self.handle, (*curr).data) };
             let next = unsafe { (*curr).next };
             if !f(&item) {
                 drop(item);
@@ -324,7 +367,12 @@ where
 
         let item = unsafe { alpm_list_nth(self.list.list, n) };
         unsafe { self.list.list = alpm_list_remove_item(self.list.list, item) };
-        let ret = unsafe { Some(T::into_alpm_list_item(self.handle, (*self.list.list).data)) };
+        let ret = unsafe {
+            Some(T::ptr_into_alpm_list_item(
+                self.handle,
+                (*self.list.list).data,
+            ))
+        };
         unsafe { free(item as _) };
         ret
     }
@@ -341,7 +389,7 @@ where
         AlpmListMut::from_parts(self.handle, item)
     }
 
-    pub fn as_alpm_list(&self) -> AlpmList<'a, T> {
+    pub fn as_list(&self) -> AlpmList<'a, T> {
         self.list
     }
 }
@@ -403,7 +451,7 @@ where
     }
 }
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Copy, Clone)]
 pub struct Iter<'a, 'b, T>
 where
     T: IntoAlpmListItem<'a, 'b>,
@@ -412,7 +460,16 @@ where
     current: *mut alpm_list_t,
 }
 
-#[derive(Debug, Copy, Clone)]
+impl<'a, 'b, T> fmt::Debug for Iter<'a, 'b, T>
+where
+    for<'c> T: IntoAlpmListItem<'a, 'c>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Iter").field("list", self.list).finish()
+    }
+}
+
+#[derive(Copy, Clone)]
 pub struct IntoIter<'a, T>
 where
     T: IntoAlpmListItem<'a, 'a>,
@@ -421,6 +478,15 @@ where
     current: *mut alpm_list_t,
 }
 
+impl<'a, T> fmt::Debug for IntoIter<'a, T>
+where
+    for<'b> T: IntoAlpmListItem<'a, 'b>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Iter").field("list", &self.list).finish()
+    }
+}
+
 impl<'a, 'b, T> Iter<'a, 'b, T>
 where
     T: IntoAlpmListItem<'a, 'b>,
@@ -437,7 +503,6 @@ where
     }
 }
 
-#[derive(Debug)]
 pub struct IntoIterMut<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
@@ -446,6 +511,15 @@ where
     current: *mut alpm_list_t,
 }
 
+impl<'a, T> fmt::Debug for IntoIterMut<'a, T>
+where
+    for<'b> T: IntoAlpmListItem<'a, 'b>,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Iter").field("list", &self.list).finish()
+    }
+}
+
 impl<'a, T> Drop for IntoIterMut<'a, T>
 where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
@@ -468,7 +542,7 @@ where
         let data = self.next_data();
 
         match data {
-            Some(data) => unsafe { Some(T::into_alpm_list_item(self.list.handle, data)) },
+            Some(data) => unsafe { Some(T::ptr_into_alpm_list_item(self.list.handle, data)) },
             None => None,
         }
     }
@@ -488,7 +562,7 @@ where
         let data = self.next_data();
 
         match data {
-            Some(data) => unsafe { Some(T::as_alpm_list_item(self.list.handle, data)) },
+            Some(data) => unsafe { Some(T::ptr_as_alpm_list_item(self.list.handle, data)) },
             None => None,
         }
     }
@@ -508,7 +582,7 @@ where
         let data = self.next_data();
 
         match data {
-            Some(data) => unsafe { Some(T::as_alpm_list_item(self.list.handle, data)) },
+            Some(data) => unsafe { Some(T::ptr_as_alpm_list_item(self.list.handle, data)) },
             None => None,
         }
     }
@@ -578,19 +652,19 @@ where
     }
 }
 
+unsafe impl<'a> AsAlpmListItemPtr<'a> for Pkg<'a> {
+    type Output = Pkg<'a>;
+
+    fn as_ptr(&self) -> *mut c_void {
+        self.pkg as *mut c_void
+    }
+}
+
 unsafe impl<'a> AsAlpmListItemPtr<'a> for Package<'a> {
     type Output = Package<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.pkg as *mut c_void
-    }
-}
-
-unsafe impl<'a> AsAlpmListItemPtr<'a> for &Package<'a> {
-    type Output = Package<'a>;
-
-    fn as_ptr(&self) -> *mut c_void {
-        self.pkg as *mut c_void
+        self.pkg.pkg as *mut c_void
     }
 }
 
@@ -602,14 +676,6 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Db<'a> {
     }
 }
 
-unsafe impl<'a> AsAlpmListItemPtr<'a> for &Db<'a> {
-    type Output = Db<'a>;
-
-    fn as_ptr(&self) -> *mut c_void {
-        self.db as *mut c_void
-    }
-}
-
 unsafe impl<'a> AsAlpmListItemPtr<'a> for Depend {
     type Output = Dep<'a>;
 
@@ -618,6 +684,14 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Depend {
     }
 }
 
+unsafe impl<'a, T: AsAlpmListItemPtr<'a>> AsAlpmListItemPtr<'a> for &T {
+    type Output = T::Output;
+
+    fn as_ptr(&self) -> *mut c_void {
+        (*self).as_ptr()
+    }
+}
+
 unsafe impl<'a> AsAlpmListItemPtr<'a> for Dep<'a> {
     type Output = Dep<'a>;
 
@@ -634,15 +708,6 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for String {
     }
 }
 
-unsafe impl<'a> AsAlpmListItemPtr<'a> for &String {
-    type Output = String;
-    const FREE: Option<unsafe extern "C" fn(_ptr: *mut c_void)> = Some(free);
-
-    fn as_ptr(&self) -> *mut c_void {
-        unsafe { strndup(self.as_bytes().as_ptr() as _, self.len()) as *mut c_void }
-    }
-}
-
 unsafe impl<'a> AsAlpmListItemPtr<'a> for &str {
     type Output = String;
     const FREE: Option<unsafe extern "C" fn(_ptr: *mut c_void)> = Some(free);
@@ -652,16 +717,8 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for &str {
     }
 }
 
-unsafe impl<'a> AsAlpmListItemPtr<'a> for &&str {
-    type Output = String;
-    const FREE: Option<unsafe extern "C" fn(_ptr: *mut c_void)> = Some(free);
-
-    fn as_ptr(&self) -> *mut c_void {
-        unsafe { strndup(self.as_bytes().as_ptr() as _, self.len()) as *mut c_void }
-    }
-}
-
 unsafe impl<'a> Push<'a> for String {}
+unsafe impl<'a> Push<'a> for Pkg<'a> {}
 unsafe impl<'a> Push<'a> for Package<'a> {}
 unsafe impl<'a> Push<'a> for Db<'a> {}
 unsafe impl<'a> Push<'a> for Depend {}
@@ -669,23 +726,23 @@ unsafe impl<'a> Push<'a> for Dep<'a> {}
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Package<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Package::new(handle, ptr as *mut alpm_pkg_t)
     }
-    unsafe fn as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Package::new(handle, ptr as *mut alpm_pkg_t)
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Group<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Group {
             inner: ptr as *mut alpm_group_t,
             handle,
         }
     }
-    unsafe fn as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Group {
             inner: ptr as *mut alpm_group_t,
             handle,
@@ -695,33 +752,33 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Group<'a> {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Depend {
     type Borrow = Dep<'b>;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Depend::from_ptr(ptr as *mut alpm_depend_t)
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Dep::from_ptr(ptr as *mut alpm_depend_t)
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Dep<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Dep::from_ptr(ptr as *mut alpm_depend_t)
     }
 
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Dep::from_ptr(ptr as *mut alpm_depend_t)
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Backup {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Backup {
             inner: ptr as *mut alpm_backup_t,
         }
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Backup {
             inner: ptr as *mut alpm_backup_t,
         }
@@ -730,7 +787,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Backup {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for OwnedFileConflict {
     type Borrow = FileConflict<'b>;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         OwnedFileConflict {
             inner: FileConflict {
                 inner: ptr as *mut alpm_fileconflict_t,
@@ -739,7 +796,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for OwnedFileConflict {
         }
     }
 
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         FileConflict {
             inner: ptr as *mut alpm_fileconflict_t,
             phantom: PhantomData,
@@ -749,7 +806,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for OwnedFileConflict {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DependMissing {
     type Borrow = DepMissing<'b>;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         DependMissing {
             inner: DepMissing {
                 inner: ptr as *mut alpm_depmissing_t,
@@ -758,7 +815,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DependMissing {
         }
     }
 
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         DepMissing {
             inner: ptr as *mut alpm_depmissing_t,
             phantom: PhantomData,
@@ -768,33 +825,33 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DependMissing {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for OwnedConflict {
     type Borrow = Conflict<'b>;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         OwnedConflict::from_ptr(ptr as *mut alpm_conflict_t)
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Conflict::from_ptr(ptr as *mut alpm_conflict_t)
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Conflict<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Conflict::from_ptr(ptr as *mut alpm_conflict_t)
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Conflict::from_ptr(ptr as *mut alpm_conflict_t)
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Db<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Db {
             db: ptr as *mut alpm_db_t,
             handle,
         }
     }
-    unsafe fn as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         Db {
             db: ptr as *mut alpm_db_t,
             handle,
@@ -804,7 +861,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Db<'a> {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DbMut<'a> {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         DbMut {
             inner: Db {
                 db: ptr as *mut alpm_db_t,
@@ -812,7 +869,7 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DbMut<'a> {
             },
         }
     }
-    unsafe fn as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         DbMut {
             inner: Db {
                 db: ptr as *mut alpm_db_t,
@@ -824,11 +881,11 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DbMut<'a> {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for &'a str {
     type Borrow = Self;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         let s = CStr::from_ptr(ptr as *mut c_char);
         s.to_str().unwrap()
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         let s = CStr::from_ptr(ptr as *mut c_char);
         s.to_str().unwrap()
     }
@@ -836,13 +893,13 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for &'a str {
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for String {
     type Borrow = &'b str;
-    unsafe fn into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
+    unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         let s = CStr::from_ptr(ptr as *mut c_char);
         let s = s.to_str().unwrap().to_string();
         free(ptr);
         s
     }
-    unsafe fn as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+    unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         let s = CStr::from_ptr(ptr as *mut c_char);
         s.to_str().unwrap()
     }
@@ -853,6 +910,16 @@ mod tests {
     use super::*;
     use crate::SigLevel;
 
+    #[test]
+    fn test_depends_list_debug() {
+        let handle = Alpm::new("/", "tests/db").unwrap();
+        let db = handle.register_syncdb("core", SigLevel::NONE).unwrap();
+        let pkg = db.pkg("linux").unwrap();
+
+        println!("{:#?}", db.pkgs());
+        println!("{:#?}", pkg.depends());
+    }
+
     #[test]
     fn test_depends_list_free() {
         let handle = Alpm::new("/", "tests/db").unwrap();
@@ -903,15 +970,29 @@ mod tests {
 
         let mut list = AlpmListMut::new(&handle);
         list.push(pkg);
-
         assert_eq!(list.first().unwrap().name(), "pacman");
+
+        let mut list = AlpmListMut::new(&handle);
+        list.push(Depend::new("a"));
+        list.push(Depend::new("b"));
+        list.push(Depend::new("c"));
+
+        let mut list = AlpmListMut::new(&handle);
+        list.push("a".to_string());
+        list.push("b".to_string());
+        list.push("c".to_string());
+
+        let mut list = AlpmListMut::new(&handle);
+        list.push("a".to_string());
+        list.push_str("b");
+        list.push_str("c");
     }
 
     #[test]
     fn test_retain() {
         let handle = Alpm::new("/", "tests/db").unwrap();
         let db = handle.register_syncdb("core", SigLevel::NONE).unwrap();
-        let mut pkgs = db.pkgs().to_list();
+        let mut pkgs = db.pkgs().to_list_mut();
         pkgs.retain(|p| p.name().starts_with('a'));
 
         assert!(!pkgs.is_empty());
@@ -919,14 +1000,22 @@ mod tests {
     }
 
     #[test]
-    fn test_as_raw_alpm_list() {
+    fn test_into_raw_alpm_list() {
         let handle = Alpm::new("/", "tests/db").unwrap();
         let db = handle.register_syncdb("core", SigLevel::NONE).unwrap();
         let pkg = db.pkg("linux").unwrap();
-        assert_eq!(handle.syncdbs().to_list().remove_list(0).len(), 1);
+        assert_eq!(handle.syncdbs().to_list_mut().remove_list(0).len(), 1);
         pkg.sync_new_version(handle.syncdbs());
-        pkg.sync_new_version(&handle.syncdbs().to_list().remove_list(0));
+        pkg.sync_new_version(&handle.syncdbs().to_list_mut().remove_list(0));
         pkg.sync_new_version(vec![db].into_iter());
         pkg.sync_new_version(vec![db].iter());
     }
+
+    #[test]
+    fn test_into_raw_alpm_list2() {
+        let mut handle = Alpm::new("/", "tests/db").unwrap();
+
+        let list = vec![Depend::new("foo")];
+        handle.set_assume_installed(list.iter()).unwrap();
+    }
 }
